var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
import { Directive, Input, Output, EventEmitter, HostListener, ReflectiveInjector, Inject } from '@angular/core';
import { DOCUMENT } from '@angular/platform-browser';
import { ConfirmationPopoverWindow } from './confirmationPopoverWindow.component';
import { ConfirmationPopoverWindowOptions } from './confirmationPopoverOptions.provider';
/**
 * All properties can be set on the directive as attributes like so (use `ConfirmationPopoverModule.forRoot()` to configure them globally):
 * ```
 * <button
 *  class="btn btn-default"
 *  mwlConfirmationPopover
 *  [title]="title"
 *  [message]="message"
 *  placement="left"
 *  (confirm)="confirmClicked = true"
 *  (cancel)="cancelClicked = true"
 *  [(isOpen)]="isOpen">
 *   Show confirm popover!
 * </button>
 * ```
 */
var ConfirmationPopover = (function () {
    /**
     * @private
     */
    function ConfirmationPopover(viewContainerRef, elm, defaultOptions, cfr, position, renderer, document //tslint:disable-line
    ) {
        this.viewContainerRef = viewContainerRef;
        this.elm = elm;
        this.defaultOptions = defaultOptions;
        this.cfr = cfr;
        this.position = position;
        this.renderer = renderer;
        this.document = document; //tslint:disable-line
        /**
         * Whether to hide the confirm button. Default `false`.
         */
        this.hideConfirmButton = false;
        /**
         * Whether to hide the cancel button. Default `false`.
         */
        this.hideCancelButton = false;
        /**
         * Whether to disable showing the popover. Default `false`.
         */
        this.isDisabled = false;
        /**
         * Will open or show the popover when changed.
         * Can be sugared with `isOpenChange` to emulate 2-way binding like so `[(isOpen)]="isOpen"`
         */
        this.isOpen = false;
        /**
         * Will emit when the popover is opened or closed
         */
        this.isOpenChange = new EventEmitter(true);
        /**
         * An expression that is called when the confirm button is clicked.
         */
        this.confirm = new EventEmitter();
        /**
         * An expression that is called when the cancel button is clicked.
         */
        this.cancel = new EventEmitter();
        /**
         * Append the element to the document body rather than the trigger element
         */
        this.appendToBody = false;
        /**
         * @private
         */
        this.popover = null;
    }
    /**
     * @private
     */
    ConfirmationPopover.prototype.ngOnInit = function () {
        this.isOpenChange.emit(false);
    };
    /**
     * @private
     */
    ConfirmationPopover.prototype.ngOnChanges = function (changes) {
        if (changes.isOpen) {
            if (changes.isOpen.currentValue === true) {
                this.showPopover();
            }
            else {
                this.hidePopover();
            }
        }
    };
    /**
     * @private
     */
    ConfirmationPopover.prototype.ngOnDestroy = function () {
        this.hidePopover();
    };
    /**
     * @private
     */
    ConfirmationPopover.prototype.onConfirm = function () {
        this.confirm.emit(null);
        this.hidePopover();
    };
    /**
     * @private
     */
    ConfirmationPopover.prototype.onCancel = function () {
        this.cancel.emit(null);
        this.hidePopover();
    };
    /**
     * @private
     */
    ConfirmationPopover.prototype.onDocumentClick = function (target) {
        if (this.popover && !this.elm.nativeElement.contains(target) && !this.popover.location.nativeElement.contains(target)) {
            this.hidePopover();
        }
    };
    /**
     * @private
     */
    ConfirmationPopover.prototype.togglePopover = function () {
        if (!this.popover) {
            this.showPopover();
        }
        else {
            this.hidePopover();
        }
    };
    /**
     * @private
     */
    ConfirmationPopover.prototype.onResize = function () {
        this.positionPopover();
    };
    ConfirmationPopover.prototype.showPopover = function () {
        var _this = this;
        if (!this.popover && !this.isDisabled) {
            var options_1 = new ConfirmationPopoverWindowOptions();
            Object.assign(options_1, this.defaultOptions, {
                title: this.title,
                message: this.message,
                onConfirm: function () {
                    _this.onConfirm();
                },
                onCancel: function () {
                    _this.onCancel();
                },
                onAfterViewInit: function () {
                    _this.positionPopover();
                }
            });
            var optionalParams = [
                'confirmText',
                'cancelText',
                'placement',
                'confirmButtonType',
                'cancelButtonType',
                'focusButton',
                'hideConfirmButton',
                'hideCancelButton',
                'popoverClass',
                'appendToBody',
                'customTemplate'
            ];
            optionalParams.forEach(function (param) {
                if (_this[param]) {
                    options_1[param] = _this[param];
                }
            });
            var componentFactory = this.cfr.resolveComponentFactory(ConfirmationPopoverWindow);
            var binding = ReflectiveInjector.resolve([{
                    provide: ConfirmationPopoverWindowOptions,
                    useValue: options_1
                }]);
            var contextInjector = this.viewContainerRef.parentInjector;
            var childInjector = ReflectiveInjector.fromResolvedProviders(binding, contextInjector);
            this.popover = this.viewContainerRef.createComponent(componentFactory, this.viewContainerRef.length, childInjector);
            if (this.appendToBody) {
                this.renderer.invokeElementMethod(this.document.body, 'appendChild', [this.popover.location.nativeElement]);
            }
            this.isOpenChange.emit(true);
        }
    };
    ConfirmationPopover.prototype.positionPopover = function () {
        if (this.popover) {
            var popoverElement = this.popover.location.nativeElement.children[0];
            var popoverPosition = this.position.positionElements(this.elm.nativeElement, popoverElement, this.placement || this.defaultOptions.placement, this.appendToBody || this.defaultOptions.appendToBody);
            this.renderer.setElementStyle(popoverElement, 'top', popoverPosition.top + "px");
            this.renderer.setElementStyle(popoverElement, 'left', popoverPosition.left + "px");
        }
    };
    ConfirmationPopover.prototype.hidePopover = function () {
        if (this.popover) {
            this.popover.destroy();
            this.popover = null;
            this.isOpenChange.emit(false);
        }
    };
    return ConfirmationPopover;
}());
__decorate([
    Input()
], ConfirmationPopover.prototype, "title", void 0);
__decorate([
    Input()
], ConfirmationPopover.prototype, "message", void 0);
__decorate([
    Input()
], ConfirmationPopover.prototype, "confirmText", void 0);
__decorate([
    Input()
], ConfirmationPopover.prototype, "cancelText", void 0);
__decorate([
    Input()
], ConfirmationPopover.prototype, "placement", void 0);
__decorate([
    Input()
], ConfirmationPopover.prototype, "confirmButtonType", void 0);
__decorate([
    Input()
], ConfirmationPopover.prototype, "cancelButtonType", void 0);
__decorate([
    Input()
], ConfirmationPopover.prototype, "focusButton", void 0);
__decorate([
    Input()
], ConfirmationPopover.prototype, "hideConfirmButton", void 0);
__decorate([
    Input()
], ConfirmationPopover.prototype, "hideCancelButton", void 0);
__decorate([
    Input()
], ConfirmationPopover.prototype, "isDisabled", void 0);
__decorate([
    Input()
], ConfirmationPopover.prototype, "isOpen", void 0);
__decorate([
    Input()
], ConfirmationPopover.prototype, "customTemplate", void 0);
__decorate([
    Output()
], ConfirmationPopover.prototype, "isOpenChange", void 0);
__decorate([
    Output()
], ConfirmationPopover.prototype, "confirm", void 0);
__decorate([
    Output()
], ConfirmationPopover.prototype, "cancel", void 0);
__decorate([
    Input()
], ConfirmationPopover.prototype, "popoverClass", void 0);
__decorate([
    Input()
], ConfirmationPopover.prototype, "appendToBody", void 0);
__decorate([
    HostListener('document:click', ['$event.target']),
    HostListener('document:touchend', ['$event.target'])
], ConfirmationPopover.prototype, "onDocumentClick", null);
__decorate([
    HostListener('click')
], ConfirmationPopover.prototype, "togglePopover", null);
__decorate([
    HostListener('window:resize')
], ConfirmationPopover.prototype, "onResize", null);
ConfirmationPopover = __decorate([
    Directive({
        selector: '[mwlConfirmationPopover]'
    }),
    __param(6, Inject(DOCUMENT))
], ConfirmationPopover);
export { ConfirmationPopover };
//# sourceMappingURL=confirmationPopover.directive.js.map